@@ -1,10 +1,17 @@
 # coding: utf-8
 
+from io import open
+import logging
+import ntpath
 import os
-import yaml
-import urlparse
-from mkdocs import utils
 
+import yaml
+
+from mkdocs import utils
+from mkdocs.compat import urlparse
+from mkdocs.exceptions import ConfigurationError
+
+log = logging.getLogger(__name__)
 
 DEFAULT_CONFIG = {
     'site_name': None,
@@ -20,13 +27,16 @@ DEFAULT_CONFIG = {
     'site_dir': 'site',
     'theme_dir': None,
 
-    # The address on which to serve the livereloading docs servers.
+    'copyright': None,
+    'google_analytics': None,
+
+    # The address on which to serve the livereloading docs server.
     'dev_addr': '127.0.0.1:8000',
 
     # If `True`, use `<page_name>/index.hmtl` style files with hyperlinks to the directory.
     # If `False`, use `<page_name>.html style file with hyperlinks to the file.
     # True generates nicer URLs, but False is useful if browsing the output on a filesystem.
-    'use_direcory_urls': True,
+    'use_directory_urls': True,
 
     # Specify a link to the project source repo to be included
     # in the documentation pages.
@@ -48,10 +58,21 @@ DEFAULT_CONFIG = {
     'include_nav': None,
     'include_next_prev': None,
 
-    # To Do
+    # PyMarkdown extension names.
+    'markdown_extensions': (),
+
+    # Determine if the site should generate a json search index and include
+    # search elements in the theme. - TODO
     'include_search': False,
+
+    # Determine if the site should include a 404.html page.
+    # TODO: Implment this. Make this None, have it True if a 404.html
+    # template exists in the theme or docs dir.
     'include_404': False,
-    'include_sitemap': False
+
+    # enabling strict mode causes MkDocs to stop the build when a problem is
+    # encountered rather than display an error.
+    'strict': False,
 }
 
 
@@ -59,18 +80,39 @@ def load_config(filename='mkdocs.yml', options=None):
     options = options or {}
     if 'config' in options:
         filename = options['config']
-    assert os.path.exists(filename), "Config file '%s' does not exist." % filename
-
-    user_config = yaml.load(open(filename, 'r'))
+    else:
+        options['config'] = filename
+    if not os.path.exists(filename):
+        raise ConfigurationError("Config file '%s' does not exist." % filename)
+    with open(filename, 'r', encoding='utf-8') as fp:
+        user_config = yaml.load(fp)
+        if not isinstance(user_config, dict):
+            raise ConfigurationError("The mkdocs.yml file is invalid. See http://www.mkdocs.org/user-guide/configuration/ for more information.")
     user_config.update(options)
     return validate_config(user_config)
 
 
 def validate_config(user_config):
     config = DEFAULT_CONFIG.copy()
+
+    theme_in_config = 'theme' in user_config
+
     config.update(user_config)
 
-    assert config['site_name'], "Config must contain 'site_name' setting."
+    if not config['site_name']:
+        raise ConfigurationError("Config must contain 'site_name' setting.")
+
+    # Validate that the docs_dir and site_dir don't contain the
+    # other as this will lead to copying back and forth on each
+    # and eventually make a deep nested mess.
+    abs_site_dir = os.path.abspath(config['site_dir'])
+    abs_docs_dir = os.path.abspath(config['docs_dir'])
+    if abs_docs_dir.startswith(abs_site_dir):
+        raise ConfigurationError(
+            "The 'docs_dir' can't be within the 'site_dir'.")
+    elif abs_site_dir.startswith(abs_docs_dir):
+        raise ConfigurationError(
+            "The 'site_dir' can't be within the 'docs_dir'.")
 
     # If not specified, then the 'pages' config simply includes all
     # markdown files in the docs dir, without generating any header items
@@ -78,10 +120,10 @@ def validate_config(user_config):
     pages = []
     extra_css = []
     extra_javascript = []
-    for (dirpath, dirnames, filenames) in os.walk(config['docs_dir']):
+    for (dirpath, _, filenames) in os.walk(config['docs_dir']):
         for filename in sorted(filenames):
             fullpath = os.path.join(dirpath, filename)
-            relpath = os.path.relpath(fullpath, config['docs_dir'])
+            relpath = os.path.normpath(os.path.relpath(fullpath, config['docs_dir']))
 
             if utils.is_markdown_file(filename):
                 # index pages should always be the first listed page.
@@ -96,6 +138,23 @@ def validate_config(user_config):
 
     if config['pages'] is None:
         config['pages'] = pages
+    else:
+        """
+        If the user has provided the pages config, then iterate through and
+        check for Windows style paths. If they are found, output a warning
+        and continue.
+        """
+        for page_config in config['pages']:
+            if isinstance(page_config, str):
+                path = page_config
+            elif len(page_config) in (1, 2, 3):
+                path = page_config[0]
+
+            if ntpath.sep in path:
+                log.warning("The config path contains Windows style paths (\\ "
+                            " backward slash) and will have comparability "
+                            "issues if it is used on another platform.")
+                break
 
     if config['extra_css'] is None:
         config['extra_css'] = extra_css
@@ -103,15 +162,23 @@ def validate_config(user_config):
     if config['extra_javascript'] is None:
         config['extra_javascript'] = extra_javascript
 
-    if config['theme_dir'] is None:
-        package_dir = os.path.dirname(__file__)
-        config['theme_dir'] = os.path.join(package_dir, 'themes', config['theme'])
+    package_dir = os.path.dirname(__file__)
+    theme_dir = [os.path.join(package_dir, 'themes', config['theme'])]
+
+    if config['theme_dir'] is not None:
+        # If the user has given us a custom theme but not a
+        # builtin theme name then we don't want to merge them.
+        if not theme_in_config:
+            theme_dir = []
+        theme_dir.insert(0, config['theme_dir'])
+
+    config['theme_dir'] = theme_dir
 
     if config['repo_url'] is not None and config['repo_name'] is None:
-        repo_host = urlparse.urlparse(config['repo_url']).netloc.lower()
+        repo_host = urlparse(config['repo_url']).netloc.lower()
         if repo_host == 'github.com':
             config['repo_name'] = 'GitHub'
-        elif repo_host == 'bitbucket.com':
+        elif repo_host == 'bitbucket.org':
             config['repo_name'] = 'Bitbucket'
         else:
             config['repo_name'] = repo_host.split('.')[0].title()
@@ -132,5 +199,6 @@ def validate_config(user_config):
     # Cannot set repo_name without setting repo_url.
     # Cannot set 'include_next_prev: true' when only one page exists.
     # Cannot set 'include_nav: true' when only one page exists.
+    # Error if any config keys provided that are not in the DEFAULT_CONFIG.
 
     return config
